//*****************************************************************************
//*
//*
//*		HttpClient.cpp
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
#include	<stdio.h>
#include	<string.h>
#include	"HttpClient.h"
#include	"System.h"


static	char   cHttpGet[] =
	" HTTP/1.0\n"
	"Accept: image/gif, image/x-xbitmap, image/jpeg, image/png, */*\r\n"
	"Accept-Language: de\r\n"
	"Accept-Charset: iso-8859-1,*,utf-8\r\n"
	"Keep-Alive: 300\r\n"
	"Connection: keep-alive\r\n";



//*****************************************************************************
//*
//*		Konstructor
//*
//*****************************************************************************
CHttpClient::CHttpClient()
{

	hTcp=0;
	pData=0;
	iError=0;
	iDataMax=0;
	iDataStop=0;
	iDataStart=0;
	iDataLength=0;
	uCookiePos=0;
	uCookieCount=0;
	iReconnectMax=2;
	iReconnectLevel=0;
	iDataState=0;
	iCookieNumber=0;
	iNewConnection=0;
	iHttpVersion=0;

	cServername[0]=0;
	memset(pCookies,0,sizeof(pCookies));
}

//*****************************************************************************
//*
//*		Destructor
//*
//*****************************************************************************
CHttpClient::~CHttpClient()
{


	if(hTcp)Disconnect();

	if(pData)delete pData;

	DeleteAllCookies();

}



//*****************************************************************************
//*
//*		Connect
//*
//*****************************************************************************
//	Baut eine Verbindung zu einem Webserver auf
//	pServerName	: Name des Servers oder eine URL (z.B. http://www.servername/xxx )
//	Ergibt:		 0 wenn ok
//				-6 wenn die Verbindung nicht geffnet wurde
//				-7 wenn der Servername falsch ist
//				-8 wenn der Servername nicht aufgelst werden konnte
int CHttpClient::Connect(const char *pServerName)
{
char	cChar;
int		i,iTurn;
IpAddr	iAddr;


	if(hTcp)Disconnect();

	if(!pServerName)
		{
		iError=HCL_WRONG_SERVERNAME;
		return HCL_WRONG_SERVERNAME;
		}

	for(i=0,iTurn=0;i<sizeof(cServername)-1;i++)
		{
		cChar=pServerName[i];

		if(cChar== 0  )break;
		if(cChar=='/' )break;
		if(cChar=='\\')break;
		if(cChar!=':' )continue;

		if(iTurn>0)break;

		for(i++;pServerName[i]=='/';i++);
		pServerName+=i;
		iTurn++;
		i--;
		}

	strcpy(cServername,pServerName);
	cServername[i]=0;

	if(!IpHostname(&iAddr,cServername))
		{
		iError=HCL_SERVER_NOT_FOUND;
		return HCL_SERVER_NOT_FOUND;
		}

	hTcp=TcpOpen(&iAddr,80,0,0);
	iError=(hTcp)? HCL_OK:HCL_NOCONNECTION;



return iError;
}


//*****************************************************************************
//*
//*		Disconnect
//*
//*****************************************************************************
//	Schliest die Verbindung zum Server
//	Ergibt:     0 wenn ok
//			   -1 Falsches Handle
int CHttpClient::Disconnect()
{

	if(!hTcp)return HCL_WRONG_HANDLE;
	TcpClose(hTcp);
	iHttpVersion=0;
	hTcp=0;


return HCL_OK;
}

//*****************************************************************************
//*
//*		Get
//*
//*****************************************************************************
//	Liest eine Datei von einer HTTP Verbindung ein, an die Datei wird ein
//	Null-Byte angehngt
//	pPath	 : Name der gewnschten Datei ohne Servernamen
//	uTimeout : Gibt die Wartezeit an bis auf das letzte Byte gewartet wird.
//	Ergibt   : Die Lnge der Daten
//			   -1  Falsches Handle
//			   -2  Fehler beim Senden
//			   -3  Fehler beim Lesen
//			   -4  Zuwenig Speicher
//			   -5  Timeout
//			   -9  Es gab zu viele Umleitungen
//			   -10 Das Ziel der Umleitung wurde nicht gefunden
//			   -11 Die Anfrage wurde zu gro
int CHttpClient::Get(const char *pPath,unsigned uTimeout)
{
char   *pNew,*pText,*pValue;
char	cGet[sizeof(cHttpGet)+512+256*HCL_COOKIES_MAX+4];
int		iLen,iNum,iVersion;
SysDW	uLastTime;



	iError		= HCL_OK;
	iDataLength	= 0;
	iDataStop	= 0;
	iDataStart	= 0;
	iDataState  = 0;


	if(iNewConnection)						// Die Verbindung neu aufbauen ?
		{
		iVersion=iHttpVersion;
		Disconnect();
		Connect(cServername);
		iHttpVersion=iVersion;
		iNewConnection=0;
		}

	if(!hTcp)
		{
		iError=HCL_WRONG_HANDLE;
		return HCL_WRONG_HANDLE;
		}


		iLen=CreateRequest("GET",pPath,cGet,sizeof(cGet));
	if(!iLen)
		{
		iError=HCL_REQUEST_TOO_LARGE;
		return HCL_REQUEST_TOO_LARGE;
		}

	if(!TcpPut(hTcp,cGet,iLen))
		{
		iError=HCL_SEND_ERROR;
		return HCL_SEND_ERROR;
		}


	if(!iDataMax)							// Einen neuen Datenspeicher allocieren
		{
		iDataMax=4096;

			pData=new char[iDataMax];
		if(!pData)
			{
			iDataMax=0;
			iError=HCL_OUT_OF_MEMORY;
			return HCL_OUT_OF_MEMORY;
			}
		}

	uLastTime = SysTickCount();

	for(;;)
		{
		   iNum =TcpGetWait(hTcp,pData+iDataLength,iDataMax-iDataLength-1,0);
		if(iNum <0)
			{
			iError=HCL_READ_ERROR;
			break;
			}
		if(iNum==0)
			{
			if(SysTickCount()-uLastTime>uTimeout)
				{
				iError=HCL_TIMEOUT;
				break;
				}
			SysSleep(50);
			continue;
			}


		uLastTime = SysTickCount();
		iDataLength+=iNum;

		if(!iDataStart)						// Wurde der Header bertragen
			{
					  pData[iDataLength]=0;
			          pText=strstr(pData,"\r\n\r\n");
			if(!pText)pText=strstr(pData,"\n\n");
			if( pText)
				{
				if(!memcmp(pData,"HTTP/",5))
					{
					iLen=strcspn(pData+5," \t\n\r");
					iDataState=atoi(pData+iLen+5);
					if(!memcmp(pData+5,"1.1",3))iHttpVersion=1;
					}

				iDataStart = pText-pData+4;

				   pValue = strstr(pData,"Set-Cookie:");
				if(pValue && pValue<pText)
					{
					AddCookie(pValue+11);
					}

					pValue=strstr(pData,"Connection:");
				if( pValue && pValue<pText)
					{
					pValue+=11;
					pValue+=strspn(pValue," \t");
					if(!memcmp(pValue,"close",5))
						{
						iNewConnection=1;
						}
					}

				   pValue = strstr(pData,"Content-Length:");
				if(pValue && pValue<pText)
					{
					   iLen = atoi(pValue+15);
					if(iLen)iDataStop=iDataStart+iLen;
					}

				if(iDataState==302)			// Umleitung
					{
					return Reconnect(uTimeout);
					}

				if(iDataState!=200 && !iDataStop)break;
				}
			}

		if(iDataStop && iDataLength>=iDataStop)break;

		if(iDataLength+256>iDataMax)		// Den Speicher vergrern
			{
			iDataMax += iDataMax/2;
			pNew  = new char[iDataMax];
			if(!pNew)
				{
				iError=HCL_OUT_OF_MEMORY;
				break;
				}

			memcpy(pNew,pData,iDataLength);
			delete pData;
			pData=pNew;
			}

		if(!iDataStop && iDataLength>10)
			{
			char cTemp[16];

			memcpy(cTemp,pData+iDataLength-10,10);
			cTemp[10]=0;
			strupr(cTemp);

			    pText=strstr(cTemp,"</HTML>");
			if( pText)
				{
				iDataStop = pText-pData;
				break;
				}
			}
		}

	if(!iDataStart)							// Wurden Daten empfangen
		{
		return iError;
		}


	pData[iDataLength]=0;



return iError;
}

//*****************************************************************************
//*
//*		Reconnect
//*
//*****************************************************************************
//	Wird aufgerufen wenn ein Server eine Anfrage weiterleitet
//	Des Ziel der Weiterleitung muss im aktuellen Header stehen
int	CHttpClient::Reconnect(unsigned uTimeout)
{
int		iRet,iLen,iVersion;
char   *pLocation,*pValue;



	if(iReconnectLevel+1>=iReconnectMax)
		{
		iError=HCL_RECONNECT_LOOP;
		return HCL_RECONNECT_LOOP;
		}

	pData[iDataStart]=0;

	    pValue=strstr(pData,"Connection:");
	if( pValue)
		{
		pValue+=11;
		pValue+=strspn(pValue," \t");
		if(!memcmp(pValue,"close",5))
			{
			char		cServer[256];

			strncpy(cServer,cServername,sizeof(cServer));
			iVersion=iHttpVersion;
			Disconnect();
			iHttpVersion=iVersion;
			if(Connect(cServer)!=HCL_OK)return iError;
			}
		}

		pLocation=strstr(pData,"Location:");
	if(!pLocation)
		{
		iError=HCL_LOCATION_NOT_FOUND;
		return HCL_LOCATION_NOT_FOUND;
		}

	pLocation+=9;
	pLocation+=strspn(pLocation," \t");
	iLen=strcspn(pLocation,";\n\r");
	pLocation[iLen]=0;


	iReconnectLevel++;
	iRet=Get(pLocation,uTimeout);
	iReconnectLevel--;

return iRet;

}

//*****************************************************************************
//*
//*		AddCookie
//*
//*****************************************************************************
//	Fgt ein Cookie in die Liste ein
//	Ergibt:	Die einfge Position bzw -1 bei einem Fehler
int	CHttpClient::AddCookie(const char *pText)
{
unsigned	u,uPos,uLen,uNum;
char	   *pTemp,*pAlloc;



	if(!pText)return -1;

	  pText+=strspn (pText," \t");
		uLen=strcspn(pText,";\n\r");
	if(!uLen)return -1;

	for(u=0,uPos=uCookiePos+1;u<HCL_COOKIES_MAX;u++,uPos++)
		{
		if(uPos>=HCL_COOKIES_MAX)uPos=0;
		if(!pCookies[uPos])break;
		}


	if(uPos>=HCL_COOKIES_MAX)uPos=0;

	pTemp=pCookies[uPos];
	pAlloc=new char[uLen+256];

	uNum=sprintf(pAlloc,"Cookie: lR_12a_cookie_rechner=%i; ",iCookieNumber);
	memcpy(pAlloc+uNum,pText,uLen);
	memcpy(pAlloc+uNum+uLen,"\r\n",3);

	iCookieNumber++;
	pCookies[uPos]=pAlloc;


	if(pTemp)delete pTemp;
	else	uCookieCount++;


return 0;
}

//*****************************************************************************
//*
//*		Post
//*
//*****************************************************************************
//	Schickt eine Nachricht zum Server
//	pPath	 : Name der gewnschten Datei ohne Servernamen
//	pMessage : Zeiger auf die Daten die geschickt werden sollen
//	iSize	 : Lnge der Daten die geschickt werden.
//			   Bei -1 wird die Lnge mit strlen(pData) ermittelt
//	uTimeout : Gibt die Wartezeit an bis auf das letzte Byte gewartet wird.
//	Ergibt   : Die Lnge der Daten
//			   -1  Falsches Handle
//			   -2  Fehler beim Senden
//			   -3  Fehler beim Lesen
//			   -4  Zuwenig Speicher
//			   -5  Timeout
//			   -9  Es gab zu viele Umleitungen
//			   -10 Das Ziel der Umleitung wurde nicht gefunden
//			   -11 Die Anfrage wurde zu gro
int CHttpClient::Post(const char *pPath,const char *pMessage,int iSize,unsigned uTimeout)
{
char   *pNew,*pText,*pValue;
char	cPost[sizeof(cHttpGet)+512+256*HCL_COOKIES_MAX+4];
int		iLen,iNum,iVersion;
SysDW	uLastTime;



	iError		= HCL_OK;
	iDataLength	= 0;
	iDataStop	= 0;
	iDataStart	= 0;
	iDataState  = 0;


	if(iNewConnection)						// Die Verbindung neu aufbauen ?
		{
		iVersion=iHttpVersion;
		Disconnect();
		Connect(cServername);
		iHttpVersion=iVersion;
		iNewConnection=0;
		}

	if(!hTcp)
		{
		iError=HCL_WRONG_HANDLE;
		return HCL_WRONG_HANDLE;
		}

	if(iSize<0)iSize=strlen(pMessage);

		iLen=CreateRequest("POST",pPath,cPost,sizeof(cPost)-64,0);
	if(!iLen)
		{
		iError=HCL_REQUEST_TOO_LARGE;
		return HCL_REQUEST_TOO_LARGE;
		}

	iLen+=sprintf(cPost+iLen,"Content-Type: application/x-www-form-urlencoded\r\n"
							 "Content-Length: %i\r\n\r\n",iSize);


	if(!TcpPut(hTcp,cPost,iLen))
		{
		iError=HCL_SEND_ERROR;
		return HCL_SEND_ERROR;
		}

	if(!TcpPut(hTcp,pMessage,iSize))
		{
		iError=HCL_SEND_ERROR;
		return HCL_SEND_ERROR;
		}


	if(!iDataMax)							// Einen neuen Datenspeicher allocieren
		{
		iDataMax=4096;

			pData=new char[iDataMax];
		if(!pData)
			{
			iDataMax=0;
			iError=HCL_OUT_OF_MEMORY;
			return HCL_OUT_OF_MEMORY;
			}
		}


	uLastTime = SysTickCount();

	for(;;)
		{
		   iNum =TcpGetWait(hTcp,pData+iDataLength,iDataMax-iDataLength-1,0);
		if(iNum <0){iError=HCL_READ_ERROR;break;}
		if(iNum==0)
			{
			if(SysTickCount()-uLastTime>uTimeout)
				{
				iError=HCL_TIMEOUT;
				break;
				}
			SysSleep(50);
			continue;
			}

		uLastTime = SysTickCount();
		iDataLength+=iNum;

		if(!iDataStart)						// Wurde der Header bertragen
			{
			pData[iDataLength]=0;
			pText=strstr(pData,"\r\n\r\n");
			if(pText)
				{
				if(!memcmp(pData,"HTTP/",5))
					{
					iLen=strcspn(pData+5," \t\n\r");
					iDataState=atoi(pData+iLen+5);
					}

				iDataStart = pText-pData+4;

				   pValue = strstr(pData,"Set-Cookie:");
				if(pValue && pValue<pText)
					{
					AddCookie(pValue+11);
					}

					pValue=strstr(pData,"Connection:");
				if( pValue && pValue<pText)
					{
					pValue+=11;
					pValue+=strspn(pValue," \t");
					if(!memcmp(pValue,"close",5))
						{
						iNewConnection=1;
						}
					}

				   pValue = strstr(pData,"Content-Length:");
				if(pValue && pValue<pText)
					{
					   iLen = atoi(pValue+15);
					if(iLen)iDataStop=iDataStart+iLen;
					}
				else{
					iDataStop=iDataStart;
					break;
					}
				}
			}

		if(iDataStop && iDataLength>=iDataStop)break;

		if(iDataLength+256<iDataMax)		// Den Speicher vergrern
			{
			iDataMax += iDataMax/2;
			pNew  = new char[iDataMax];
			if(!pNew)
				{
				iError=HCL_OUT_OF_MEMORY;
				break;
				}

			memcpy(pNew,pData,iDataLength);
			delete pData;
			pData=pNew;
			}
		}


	if(!iDataStart)							// Wurden Daten empfangen
		{
		return iError;
		}


	pData[iDataLength]=0;



return iError;
}

//*****************************************************************************
//*
//*		CreateRequest
//*
//*****************************************************************************
//	Erzeugt einen HTTP Request
//	pType	: Art des Requestes ("GET","POST", ... )
//	pPath	: Pfad des oder URL
//	pData	: Zielspeicher fr den Request
//	iMax	: Gre des Zielspeichers
//	Ergibt die Lnge des Requestes bzw 0 bei eine Fehler
int	CHttpClient::CreateRequest(const char *pType,const char *pPath,char *pData,int  iMax,int iAddCr)
{
int		i,iLen,iNum;


	   iLen=strlen(pType);
	if(iLen<=0)return 0;
	if(iLen+1>iMax)return 0;

	memcpy(pData,pType,iLen);
	pData[iLen]=' ';
	iLen++;


	   iNum=strlen(pPath);
	if(iNum<=0){iNum=1;pPath="/";}
	if(iLen+iNum>iMax)return 0;


	memcpy(pData+iLen,pPath,iNum);			// URL Pfad anhngen
	iLen+=iNum;

	iNum=strlen(cServername);
	if((int)(iLen+iNum+8+sizeof(cHttpGet))>iMax)return 0;
	memcpy(pData+iLen,cHttpGet ,sizeof(cHttpGet));
	pData[iLen+8]+=iHttpVersion;
	iLen+=sizeof(cHttpGet)-1;

	memcpy(pData+iLen,"Host: ",6);			// Servernamen anhngen
	iLen+=6;
	memcpy(pData+iLen,cServername,iNum);
	iLen+=iNum;
	memcpy(pData+iLen,"\r\n",2);
	iLen+=2;

	for(i=0;i<HCL_COOKIES_MAX;i++)			// Cookies anhngen
		{
		if(!pCookies[i])continue;
		iNum=strlen(pCookies[i]);
		if(iLen+iNum>iMax)return 0;
		memcpy(pData+iLen,pCookies[i],iNum);
		iLen+=iNum;
		}

	if(iAddCr)								// Den Block abschlieen ?
		{
		if(iLen+2>iMax)return 0;
		memcpy(pData+iLen,"\r\n",2);
		iLen+=2;
		}



return iLen;
}


//*****************************************************************************
//*
//*		DeleteAllCookies
//*
//*****************************************************************************
//	Lscht alle Cookies
//	Ergibt die Anzahl der gelschten Cookies
int CHttpClient::DeleteAllCookies()
{
int		i,iCount=0;


	for(i=0;i<HCL_COOKIES_MAX;i++)
		{
		if(pCookies[i])continue;
		delete pCookies[i];
		pCookies[i]=0;
		iCount++;
		}

	uCookieCount=0;

return iCount;
}


//*****************************************************************************
//*
//*		SaveHtml
//*
//*****************************************************************************
//	Speichert die HTML Daten
//	pPath	: Pfad der Html-Datei
//	Ergibt 0 wenn ok
int CHttpClient::SaveHtml(const char *pPath)
{
FILE	*pFile;
int		 iOk;


	if(!iDataStart || iDataLength<=iDataStart)return HCL_NO_DATA;

		pFile=fopen(pPath,"wb");
	if(!pFile)return HCL_FILE_ERROR;

	iOk=fwrite(pData+iDataStart,iDataLength-iDataStart,1,pFile);
	fclose(pFile);

return (iOk)? HCL_OK:HCL_FILE_ERROR;
}



//*****************************************************************************
//*
//*		SaveAll
//*
//*****************************************************************************
//	Speichert die HTML Daten
//	pPath	: Pfad der Html-Datei
//	Ergibt 0 wenn ok
int CHttpClient::SaveAll(const char *pPath)
{
FILE	*pFile;
int		 iOk;


	if(!iDataStart || iDataLength<=0)return HCL_NO_DATA;

		pFile=fopen(pPath,"wb");
	if(!pFile)return HCL_FILE_ERROR;

	iOk=fwrite(pData,iDataLength,1,pFile);
	fclose(pFile);

return (iOk)? HCL_OK:HCL_FILE_ERROR;
}
