//*****************************************************************************
//*
//*
//*		HttpClient.h
//*
//*
//*****************************************************************************
#ifndef 	__HTTPCLIENT_H__
#define 	__HTTPCLIENT_H__

#include	"SocketUtility.h"


#define 	HCL_COOKIES_MAX			 4	// Maximale anzahl de Cookies

#define 	HCL_OK					 0	// Alles ok
#define 	HCL_WRONG_HANDLE		-1	// Falsches Handle
#define 	HCL_SEND_ERROR			-2	// Fehler beim Senden
#define 	HCL_READ_ERROR		    -3	// Fehler beim Lesen
#define 	HCL_OUT_OF_MEMORY	    -4	// Zuwenig Speicher
#define 	HCL_TIMEOUT				-5	// Timeout
#define 	HCL_NOCONNECTION		-6	// Timeout
#define 	HCL_WRONG_SERVERNAME	-7	// Falscher Servername
#define 	HCL_SERVER_NOT_FOUND	-8	// Der Servername konnte nicht aufgelst werden
#define 	HCL_RECONNECT_LOOP		-9	// Es gab zu viele Umleitungen	
#define 	HCL_LOCATION_NOT_FOUND	-10	// Das Ziel der Umleitung wurde nicht gefunden
#define 	HCL_REQUEST_TOO_LARGE	-11	// Die Anfrage wurde zu gro
#define 	HCL_FILE_ERROR			-12	// Die Daten konnten nicht geschrieben werden
#define 	HCL_NO_DATA				-13	// Es sind keine Daten vorhanden

//*****************************************************************************
//*
//*		CHttpClient
//*
//*****************************************************************************
class CHttpClient
{
public:

    CHttpClient();
   ~CHttpClient();


	int			Disconnect();
	int			Connect(const char *pServerName);
	int			Get (const char *pPath,unsigned uTimeout=5000);
	int			Post(const char *pPath,const char *pMessage,int iSize=-1,unsigned uTimeout=5000);
	int			AddCookie(const char *pText);
	int			DeleteAllCookies();
	int			SaveHtml(const char *pPath);
	int			SaveAll(const char *pPath);

	
	const char *Data();				// Zeiger auf Daten ohne Header
	const char *FullData();			// Zeiger auf Daten mit Header
	int			Start();			// Gre des Headers
	int			Length();			// Lnge mit Header
	int			DataLength();		// Lnge ohne Header
	int			State();			// Ergibt den letzten HTTP Rckgabewert

protected:
	

	int			Reconnect(unsigned uTimeout=5000);
	int			CreateRequest(const char *pType,const char *pPath,char *pData,int iMax,int iAddCr=1);
	
	TcpHandle	hTcp;				// TCP-Handle fr Serververbindung
	int			iError;				// Aktuelle Fehlernummer
	char       *pData;				// Zeiger auf Datenspeicher
	int			iDataMax;			// Gre des Datenspeichers
	int			iDataStart;			// Begin der HTTP-Daten im Datenspeicher
	int			iDataStop;			// Vom Server angegebene Lnge der Daten 
	int			iDataLength;		// Empfangene Lnge der Daten 
	int			iDataState;			// Empfangener Status
	int			iHttpVersion;		// Nummer der Http-Version
	int			iReconnectMax;		// Maximale Umleitungs Ebene
	int			iReconnectLevel;	// Aktuelle Umleitungs Ebene
	int			iNewConnection;		// Beim nchstenmal neu verbinden
	unsigned	uCookiePos;			// Aktuelle Cookie einfge Position
	unsigned	uCookieCount;		// Anzahl der einfgen Cookies

	int			iCookieNumber;		// Fortlaufende Cookienummer
	char	   *pCookies[HCL_COOKIES_MAX];	// Zeiger auf Cookiedaten
	char		cServername[256];	// Name des Servers		

};


//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************
inline	const char *CHttpClient::Data(){return (iDataStart)? pData+iDataStart:0;}
inline	const char *CHttpClient::FullData(){return (iDataLength)? pData:0;}
inline	int			CHttpClient::Start(){return iDataStart;}
inline	int			CHttpClient::Length(){return iDataLength;}
inline	int			CHttpClient::DataLength(){return (iDataStart)? iDataLength-iDataStart:0;}
inline	int			CHttpClient::State(){return iDataState;}

#endif 	