/*****************************************************************************
/*
/*
/*		System.h
/*
/*
/*****************************************************************************/
#ifndef 	__SYSTEM_H__
#define 	__SYSTEM_H__


#ifdef 		_DEBUG
	#define HEAP_DEBUG
#else
	#ifdef	HEAP_DEBUG
	#undef 	HEAP_DEBUG
	#endif
#endif




#ifdef 	__cplusplus
extern "C"{
#endif

#ifdef		RTTARGET


	#define 	SYS_HAS_SHORT_NAME	1

	#ifndef		TRUE
	#define 	TRUE		1
	#endif

	#ifndef		FALSE
	#define 	FALSE		0
	#endif

	#ifndef		SEEK_SET
	#define 	SEEK_SET	0
	#endif

	#ifndef		SEEK_CUR
	#define 	SEEK_CUR	1
	#endif

	#ifndef		SEEK_END
	#define 	SEEK_END	2
	#endif

	#ifndef		MAX_PATH
	#define 	MAX_PATH	256
	#endif

	#ifndef WINBASEAPI
	#define WINBASEAPI __declspec(dllimport)
	#endif

	WINBASEAPI int		__stdcall HeapDestroy(void*  hHeap);
	WINBASEAPI void*	__stdcall HeapAlloc  (void*  hHeap,unsigned long dwFlags,unsigned long dwBytes);
	WINBASEAPI int		__stdcall HeapFree   (void*  hHeap,unsigned long dwFlags,void* lpMem);
	WINBASEAPI void*	__stdcall HeapReAlloc(void*  hHeap,unsigned long dwFlags,void *lpMem,unsigned long dwBytes);
	WINBASEAPI void*	__stdcall HeapCreate (unsigned long flOptions,unsigned long dwInitialSize,unsigned long dwMaximumSize);

	typedef	int BOOL;

#endif




#define 	SYS_READ		0x0001
#define 	SYS_WRITE		0x0002
#define 	SYS_CREATE		0x0004
#define 	SYS_TRUNC		0x0008
#define 	SYS_DIR			0x0010
#define 	SYS_HIDDEN		0x0020
#define 	SYS_ARCHIVE		0x0040
#define 	SYS_SYSTEM		0x0080
#define 	SYS_VOLUME 		0x0100
#define 	SYS_SHARE_RD 	0x0400
#define 	SYS_SHARE_WR 	0x0800

#define 	SYS_GLOBAL		0x80000000

typedef		int				 SysFile;
typedef		int              SysFind;
typedef		void			*SysSemaphore;
typedef		unsigned	     SysDW;
typedef		unsigned		 SysUtc;
typedef		void*			 SysHeap;

typedef	struct
	{
	SysDW	dwAttr;
	SysDW	dwSize;
	SysUtc	dwTimeCreate;
	SysUtc	dwTimeAccess;
	SysUtc	dwTimeWrite;
	char	cName[256];
	#if SYS_HAS_SHORT_NAME
	char	cShortName[16];
	#endif
	}SysFindStruct;

typedef	struct
	{
	SysDW	dwFreeUnits;
	SysDW	dwTotalUnits;
	SysDW	dwBytesPerSector;
	SysDW	dwSectorsPerUnit;
	SysDW	dwSerialNumber;
	char	cName[16];
	char	cFsType[16];
	}SysVolumeInfo;

	/* File Functions */

	SysFile	SysOpen  (const char *pFilename,unsigned uFlags);
	int		SysClose (SysFile hHandle);
	int		SysFlush (SysFile hHandle);
	int		SysUnmap (SysFile hHandle);
	SysFile	SysMap   (SysFile hHandle,SysDW dwOffset,SysDW dwSize,SysDW dwFlags);
	int		SysLock  (SysFile hHandle,SysDW dwOffset,SysDW dwSize);
	int		SysUnlock(SysFile hHandle,SysDW dwOffset,SysDW dwSize);
	long	SysSeek  (SysFile hHandle,SysDW dwOffset,int   iType);
	long	SysTell  (SysFile hHandle);
	long	SysRead  (SysFile hHandle,void *pBuffer ,SysDW dwSize);
	long	SysWrite (SysFile hHandle,void *pBuffer ,SysDW dwSize);
	long	SysGetFileSize(SysFile hHandle);
	int		SysGetFileInfo(SysFile hHandle,SysFindStruct *pFind);

	int		SysMkdir (const char *pFilename);
	int		SysRmdir (const char *pFilename);
	int		SysDelete(const char *pFilename);
	int		SysRename(const char *pFilename,const char *pNewFilename);

	int		SysGetAttributes(const char *pFilename);
	int 	SysSetAttributes(const char *pFilename,SysDW dwAttrib);

	int 	SysGetFileTime(SysFile hHandle,SysDW *dwWrite,SysDW *dwCreate,SysDW *dwAccess);
	int 	SysSetFileTime(SysFile hHandle,SysDW  dwWrite,SysDW  dwCreate,SysDW  dwAccess);
	int		SysSetFileSize(SysFile hHandle,SysDW  dwSize);

	int		SysGetVolumeInfo(const char *pFilename,SysVolumeInfo *pInfo,int iCalcFree);


	/* Suchfunktionen */

	SysFind	SysFindFirst(const char *Filename,SysFindStruct *pFind);
	int		SysFindNext (SysFind hHandle     ,SysFindStruct *pFind);
	int		SysFindClose(SysFind hHandle);


	/* Mutex Functions */

	SysSemaphore	SysSemaphoreCreate  ();
	int				SysSemaphoreDelete  (SysSemaphore hSema);
	int				SysSemaphoreUnlock  (SysSemaphore hSema);
	int				SysSemaphoreLock    (SysSemaphore hSema);
	int				SysSemaphoreLockWait(SysSemaphore hSema,SysDW dwMillisec);



	/* Thread Funktions */

	#ifndef WIN32
	void		SysSleep(SysDW dwMillisec);
	#endif
	SysDW		SysThreadStart( void( _cdecl *StartProc)(void*),
								SysDW		  dwStackSize,
								void		 *pParam,
								int			  iPriority,
								const char   *pName);

	/* Time Funktions  */

	int		SysTimeConvertDU(SysDW dwDosTime ,SysDW *pUnixTime);
	int		SysTimeConvertUD(SysDW dwUnixTime,SysDW *pDosTime );
	int		SysTimeConvertUL(SysDW dwUnixTime,SysDW *pLongTime);
	int		SysTimeConvertLU(void *pLongTime ,SysDW *pUnixTime);
	#ifndef WIN32
	SysDW	SysTickCount();
	#endif

	/* Memory Funktionen */

	#ifdef 	HEAP_DEBUG
	void	   *HeapReAllocD(void *hHeap,SysDW dwFlags,void *pMem,SysDW dwBytes,int iLine,const char *pFile);
	void	   *HeapAllocD  (void *hHeap,SysDW dwFlags,           SysDW dwBytes,int iLine,const char *pFile);
	int			HeapFreeD   (void *hHeap,SysDW dwFlags,void *pMem              ,int iLine,const char *pFile);
	int			HeapPrint();
	#define 	SysHeapReAlloc(h,p,s)	HeapReAllocD(h,0,p,s,__LINE__,__FILE__)
	#define 	SysHeapAlloc(h,s)		HeapAllocD(h,0,s,__LINE__,__FILE__)
	#define 	SysHeapFree(h,p)		HeapFreeD(h,0,p,__LINE__,__FILE__)
	#else
	#define 	SysHeapReAlloc(h,p,s)	HeapReAlloc(h,0,p,s)
	#define 	SysHeapAlloc(h,s)		HeapAlloc(h,0,s)
	#define 	SysHeapFree(h,p)		HeapFree(h,0,p)
	#define 	HeapPrint()
	#endif
	#define 	SysHeapCreate(s)		HeapCreate(0,s,s)
	#define 	SysHeapDestroy(h)		HeapDestroy(h)

	/* Error Funktions  */

	#define 	SYS_ERR_NONE				 0
	#define 	SYS_ERR_PARAM				-1
	#define 	SYS_ERR_INVALID_FILENAME	-2
	#define 	SYS_ERR_FILE_NOT_FOUND		-3
	#define 	SYS_ERR_ACCESS_DENIED		-4
	#define 	SYS_ERR_NO_HANDLES			-5
	#define 	SYS_ERR_UNKNOWN				-6
	#define 	SYS_ERR_PATH_NOT_FOUND		-7
	#define 	SYS_ERR_DISKFULL			-8

#ifdef WIN32

	#define 	SysSleep(i)		Sleep(i)
	#define 	SysTickCount()	GetTickCount()
	#include	<windows.h>

#endif


#ifdef 	__cplusplus
}
#endif

#endif
